'
' Hardware SPI for the PIC18F2xKxx series.
' Amicus_HSPI.inc
' Based on the Amicus18_SPI.inc code.
' Modified by A. Freixanet 17/02/2016
' Tested and fixed the Read_SPI2() command.
' Added the ANSEL code for SPI2.
'
' 20/02/2016
' Added the SPI_WriteRead() and SPI2_WriteRead() commands.
' Added some minor codes and definitions.
' 5 Chip Select pins are pre-declared to prepare the Chip Select iddle State.
' Declare the Chip Select pins and the ANSEL bit in the main program.
' Example:
'    $define SPI_CS0_Pin   PORTA.1
'    $define CS0_Ansel_Pin ANSELA.1
'   Include "Amicus_HSPI.inc"
'
$ifndef __SPI_INC_
$define __SPI_INC_
'
'-------------------------------------------------------------------------------
    GoTo _SPI_MAIN_
'-------------------------------------------------------------------------------
' The Chip Select Pins must to be declared in the main program like this:
' Declare the Chip Select pin.
'    $define SPI_CS0_Pin   PORTC.4
' Declare correspondent bit in the ANSEL register.
'    $define CS0_Ansel_Pin ANSELC.4

$ifdef SPI_CS0_Pin
'    $define ClrCS0 Clear SPI_CS0_Pin
'    $define SetCS0 Set SPI_CS0_Pin
    $define ClrCS0 Low SPI_CS0_Pin
    $define SetCS0 High SPI_CS0_Pin
$endif

$ifdef SPI_CS1_Pin
'    $define ClrCS1 Clear SPI_CS1_Pin
'    $define SetCS1 Set SPI_CS1_Pin
    $define ClrCS1 Low SPI_CS1_Pin
    $define SetCS1 High SPI_CS1_Pin
$endif

$ifdef SPI_CS2_Pin
'    $define ClrCS2 Clear SPI_CS2_Pin
'    $define SetCS2 Set SPI_CS2_Pin
    $define ClrCS2 Low SPI_CS2_Pin
    $define SetCS2 High SPI_CS2_Pin
$endif

$ifdef SPI_CS3_Pin
'    $define ClrCS3 Clear SPI_CS3_Pin
'    $define SetCS3 Set SPI_CS3_Pin
    $define ClrCS3 Low SPI_CS3_Pin
    $define SetCS3 High SPI_CS3_Pin
$endif

$ifdef SPI_CS4_Pin
'    $define ClrCS4 Clear SPI_CS4_Pin
'    $define SetCS4 Set SPI_CS4_Pin
    $define ClrCS4 Low SPI_CS4_Pin
    $define SetCS4 High SPI_CS4_Pin
$endif

'-------------------------------------------------------------------------------
' SPI peripheral library for Amicus18
'
$define SMPEND        %10000000          ' Input data sample at end of data out
$define SMPMID        %00000000          ' Input data sample at middle of data out

$define MODE_00       %00000000          ' Setting for SPI bus Mode 0,0
$define MODE_01       %00000001          ' Setting for SPI bus Mode 0,1
$define MODE_10       %00000010          ' Setting for SPI bus Mode 1,0
$define MODE_11       %00000011          ' Setting for SPI bus Mode 1,1

' SSPCON1 Register
$define SSPENB        %00100000          ' Enable serial port and configures SCK, SDO, SDI

$define SPI_FOSC_4    %00000000          ' SPI Master mode, clock = Fosc/4
$define SPI_FOSC_16   %00000001          ' SPI Master mode, clock = Fosc/16
$define SPI_FOSC_64   %00000010          ' SPI Master mode, clock = Fosc/64
$define SPI_FOSC_TMR2 %00000011          ' SPI Master mode, clock = TMR2 output/2
$define SLV_SSON      %00000100          ' SPI Slave mode, /SS pin control enabled
$define SLV_SSOFF     %00000101          ' SPI Slave mode, /SS pin control disabled

'-----------------------------------------------------------------------------------
' Macro Name    : OpenSPI
' Return Value  : None
' Parameters    : SSP peripheral setup values
' Notes         : Sets up the SSP module for master or slave operation
'
' Syntax OpenSPI(SPI_FOSC_16, MODE_00  , SMPMID)
' Syntax OpenSPI pSyncMode  , pBusMode , pSmpPhase
'
$define OpenSPI(pSyncMode,pBusMode,pSmpPhase) _OpenSPI pSyncMode, pBusMode, pSmpPhase

_OpenSPI Macro- pSyncMode, pBusMode, pSmpPhase

$if (_device = _18F23K22) Or (_device = _18F24K22) Or (_device = _18F25K22) Or (_device = _18F26K22)

    SSPSTAT = SSPSTAT & $3F         ' Power on state
    SSPCON1 = $00                   ' Power on state
    #if(pSyncMode != 0)
        Movlw (pSyncMode)
        Iorwf SSPCON1,F             ' Select serial mode
    #endif
    #if(pSmpPhase != 0)
        Movlw (pSmpPhase)
        Iorwf SSPSTAT,F             ' Select data input sample phase
    #endif

    #if(pBusMode == 0)              ' SPI bus mode 0,0
        #if((pSyncMode & 3) == 3)   ' SPI Master mode, clock = TMR2 output/2 ?
            SSPSTATbits_CKE = 0     ' Data transmitted on falling edge
            SSPCON1bits_CKP = 0     ' Clock idle state low
        #else
            SSPSTATbits_CKE = 1     ' Data transmitted on rising edge
        #endif
    #endif
    #if(pBusMode == 2)              ' SPI bus mode 1,0
        SSPSTATbits_CKE = 1         ' Data transmitted on falling edge
        SSPCON1bits_CKP = 1         ' Clock idle state high
    #endif
    #if(pBusMode == 3)              ' SPI bus mode 1,1
        SSPCON1bits_CKP = 1         ' Clock idle state high
    #endif

    #if(pSyncMode == 4)             ' Slave mode with /SS enable
      TRISC.3 = 1                   ' Define clock pin as input
      TRISA.5 = 1                   ' Define /SS pin as input
      WREG = BSR                    ' Save the current RAM bank
      BSR = 15                      ' Point to the last RAM bank
      Bcf ANSELA,4                  ' Set /SS pin as digital
      BSR = WREG                    ' Point back to the RAM bank we were in before moving it
    #endif

    #if(pSyncMode == 5)             ' Slave mode without /SS enable
      TRISC.3 = 1                   ' Define clock pin as input
    #else                           ' Master mode, define clock pin as output
      TRISC.3 = 0                   ' Define clock pin as output
    #endif

    TRISC.4 = 1                     ' Define SDI pin as input
    TRISC.5 = 0                     ' Define SDO pin as output

    PIR1bits_SSPIF = 0              ' Clear the interrupt flag
    PIE1bits_SSPIE = 0              ' Disable SPI1 interrupt 

    SSPCON1 = SSPCON1 | SSPENB      ' Enable synchronous serial port
'-------------------------------------------------------------------------------
$endif

' 18F25K20 code
$if (_device = _18F23K20) Or (_device = _18F24K20) Or (_device = _18F25K20) Or (_device = _18F26K20)
'-------------------------------------------------------------------------------
    SSPSTAT = SSPSTAT & $3F         ' Power on state
    SSPCON1 = $00                   ' Power on state
    #if(pSyncMode != 0)
        Movlw (pSyncMode)
        Iorwf SSPCON1,F             ' Select serial mode
    #endif
    #if(pSmpPhase != 0)
        Movlw (pSmpPhase)
        Iorwf SSPSTAT,F             ' Select data input sample phase
    #endif

    #if(pBusMode == 0)              ' SPI bus mode 0,0
        #if((pSyncMode & 3) == 3)   ' SPI Master mode, clock = TMR2 output/2 ?
            SSPSTATbits_CKE = 0     ' Data transmitted on falling edge
            SSPCON1bits_CKP = 0     ' Clock idle state low
        #else
            SSPSTATbits_CKE = 1     ' Data transmitted on rising edge
        #endif
    #endif
    #if(pBusMode == 2)              ' SPI bus mode 1,0
        SSPSTATbits_CKE = 1         ' Data transmitted on falling edge
        SSPCON1bits_CKP = 1         ' Clock idle state high
    #endif
    #if(pBusMode == 3)              ' SPI bus mode 1,1
        SSPCON1bits_CKP = 1         ' Clock idle state high
    #endif

    #if(pSyncMode == 4)             ' Slave mode with /SS enable
      TRISC.3 = 1                   ' Define clock pin as input
      TRISA.5 = 1                   ' Define /SS pin as input
      ANSEL.4 = 0                   ' Set /SS pin as digital
    #endif

    #if(pSyncMode == 5)             ' Slave mode without /SS enable
      TRISC.3 = 1                   ' Define clock pin as input
    #else                           ' Master mode, define clock pin as output
      TRISC.3 = 0                   ' Define clock pin as output
    #endif

    TRISC.4 = 1                     ' Define SDI pin as input
    TRISC.5 = 0                     ' Define SDO pin as output

    PIR1bits_SSPIF = 0              ' Clear the interrupt flag
    PIE1bits_SSPIE = 0              ' Disable SPI1 interrupt 
    
    SSPCON1 = SSPCON1 | SSPENB      ' Enable synchronous serial port
$endif
Endm

'-----------------------------------------------------------------------------------
' Macro Name     : WriteSPI
' Return Value   : Input data can be picked up by reading SSPBUF straight after the WriteSPI macro
' Parameter      : Single 8-bit variable for SPI bus.
' Description    : Write a single byte (8-bits) to the SPI bus.
'
$define WriteSPI(pDataOut) '
    WREG = pDataOut '
    _WriteSPI

_WriteSPI Macro-
    GoSub __WriteSPI
Endm

#ifMacro- _WriteSPI
__WriteSPI:
    SSPSTATBits_BF = 0              ' Empty the SSPBUF buffer flags
    PIR1bits_SSPIF = 0              ' Clear the interrupt flag
    SSPBUF = WREG                   ' Place the data to send into SSPBUF
    Btfss PIR1bits_SSPIF            ' \
    Bra $ - 2                       ' / Wait until cycle complete
    WREG = SSPBUF                   ' Read SSPBUF into WREG
    Return
#endIfMacro-

'-----------------------------------------------------------------------------------
' Macro Name     : WriteReadSPI
' Return Value   : Input data can be picked up by reading SSPBUF straight after the WriteSPI macro
' Parameter      : Single 8-bit variable for SPI bus.
' Description    : Write and read a single byte (8-bits) to the SPI bus.
'
$define WriteReadSPI(pDataOut) _WriteReadSPI pDataOut

_WriteReadSPI Macro- pDataOut
    #if (Prm_1 != Num8) && (Prm_1 != Byte)
        #error "_WriteReadSPI Parameter 1 must be constant value or a Byte"
        Exitm
    #else
        #if (Prm_1 == Byte)
        Byte_Wreg pDataOut
        #endif
        #if (Prm_1 == Num8)
        Num_Wreg pDataOut
        #endif  
    #endif
    GoSub __WriteReadSPI_Sub
    return_Wreg       
Endm

#ifMacro- _WriteReadSPI
__WriteReadSPI_Sub:
    SSPSTATBits_BF = 0              ' Empty the SSPBUF buffer flags
    PIR1bits_SSPIF = 0              ' Clear the interrupt flag
    SSPBUF = WREG                   ' Place the data to send into SSPBUF
    Btfss PIR1bits_SSPIF            ' \
    Bra $ - 2                       ' / Wait until cycle complete
    WREG = SSPBUF                   ' Read SSPBUF into WREG
    Return
#endIfMacro-

'-----------------------------------------------------------------------------------
' Macro Name    : ReadSPI
' Return Value  : Contents of SSPBUF register
' Parameters    : None
' Description   : Read a single byte from the SPI bus.
'
$define ReadSPI() _ReadSPI

_ReadSPI Macro- \Byte
    GoSub __ReadSPI
    Return_Wreg
Endm

#ifMacro- _ReadSPI
__ReadSPI:
    WREG = SSPBUF                   ' Empty the SSPBUF buffer and clear flags
    PIR1bits_SSPIF = 0              ' Clear interrupt flag
    SSPBUF = $FF                    ' Initiate bus cycle
    Btfss PIR1bits_SSPIF            ' \
    Bra $ - 2                       ' / Wait until cycle complete
    WREG = SSPBUF
    Return
#endIfMacro-

'-----------------------------------------------------------------------------------

$define SPI_Write(pDataOut) WriteSPI(pDataOut)
$define SPI_WriteRead(pDataOut) WriteReadSPI(pDataOut)
$define SPI_Read() ReadSPI()
$define SPI_Open(pSyncMode,pBusMode,pSmpPhase) OpenSPI(pSyncMode,pBusMode,pSmpPhase)
$define SPI_DataReady() SSPSTATbits_BF
$define SPI_Close() SSPCON1 = SSPCON1 & $DF

$define SPI1_Write(pDataOut) WriteSPI(pDataOut)
$define SPI1_WriteRead(pDataOut) WriteReadSPI(pDataOut)
$define SPI1_Read() ReadSPI()
$define SPI1_Open(pSyncMode,pBusMode,pSmpPhase) OpenSPI(pSyncMode,pBusMode,pSmpPhase)
$define SPI1_DataReady() SSPSTATbits_BF
$define SPI1_Close() SSPCON1 = SSPCON1 & $DF
                                                    
$define WriteSPI1(pDataOut) WriteSPI(pDataOut)
$define WriteReadSPI1(pDataOut) WriteReadSPI(pDataOut)
$define ReadSPI1() ReadSPI()
$define OpenSPI1(pSyncMode,pBusMode,pSmpPhase) OpenSPI(pSyncMode,pBusMode,pSmpPhase)
$define DataReadySPI1() SSPSTATbits_BF
$define CloseSPI1() SSPCON1 = SSPCON1 & $DF

'***************************************************************
' Amicus18 Revision 2
'***************************************************************
$if (_device = _18F23K22) Or (_device = _18F24K22) Or (_device = _18F25K22) Or (_device = _18F26K22) 

'-----------------------------------------------------------------------------------
' CloseSPI2

$define CloseSPI2() SSP2CON1 = SSP2CON1 & $DF

'-----------------------------------------------------------------------------------
' DataReadySPI2: Test if SSP2BUF register is full

$define DataReadySPI2() SSP2STATbits_BF

'-----------------------------------------------------------------------------------
' Macro Name    : OpenSPI2
' Return Value  : None
' Parameters    : SSP2 peripheral setup values
' Notes         : Sets up the SSP2 module for master or slave operation
'
' Syntax OpenSPI2(SPI_FOSC_16, MODE_00  , SMPMID)
' Syntax OpenSPI2 pSyncMode  , pBusMode , pSmpPhase
'
$define OpenSPI2(pSyncMode,pBusMode,pSmpPhase) _OpenSPI2 pSyncMode, pBusMode, pSmpPhase

_OpenSPI2 Macro- pSyncMode, pBusMode, pSmpPhase
    SSP2STAT = SSP2STAT & $3F       ' Power on state
    SSP2CON1 = $00                  ' Power on state
    #if(pSyncMode != 0)
        Movlw (pSyncMode)
        Iorwf SSP2CON1,F            ' Select serial mode
    #endif
    #if(pSmpPhase != 0)
        Movlw (pSmpPhase)
        Iorwf SSP2STAT,F            ' Select data input sample phase
    #endif

    #if(pBusMode == 0)              ' SPI bus mode 0,0
        SSP2STATbits_CKE = 1        ' Data transmitted on rising edge
    #endif
    #if(pBusMode == 2)              ' SPI bus mode 1,0
        SSP2STATbits_CKE = 1        ' Data transmitted on falling edge
        SSP2CON1bits_CKP = 1        ' Clock idle state high
    #endif
    #if(pBusMode == 3)              ' SPI bus mode 1,1
        SSP2CON1bits_CKP = 1        ' Clock idle state high
    #endif

    #if(pSyncMode == 4)             ' Slave mode with /SS enable
      ANSELB.0 = 0                  ' Pin RB0 digital for SS2
      ANSELB.1 = 0                  ' Pin RB1 digital for Clock
      TRISB.1 = 1                   ' Define clock pin as input
      TRISB.0 = 1                   ' Define /SS2 pin as input
    #endif

    #if(pSyncMode == 5)             ' Slave mode without /SS enable
      ANSELB.1 = 0                  ' Pin RB1 digital for Clock
      TRISB.1 = 1                   ' Define clock pin as input
    #else                           ' Master mode, define clock pin as output
      ANSELB.1 = 0                  ' Pin RB1 digital for Clock
      TRISB.1 = 0                   ' Define clock pin as output
    #endif

    ANSELB.2 = 0                    ' Pin SDI2 as digital
    ANSELB.3 = 0                    ' Pin SDO2 as digital
    TRISB.2 = 1                     ' Define SDI2 pin as input
    TRISB.3 = 0                     ' Define SDO2 pin as output

    PIR3bits_SSP2IF = 0             ' Clear the interrupt flag
    PIE3bits_SSP2IE = 0             ' Disable the SPI2 interrupt
    
    SSP2CON1.5 = 1                  ' Enable synchronous serial2 port
Endm
'-----------------------------------------------------------------------------------
' Macro Name     : WriteSPI2
' Return Value   : Input data can be picked up by reading SSP2BUF straight after the WriteSPI2 macro
' Parameter      : Single 8-bit variable for SPI2 bus.
' Description    : Write a single byte (8-bits) to the SPI2 bus.
'
$define WriteSPI2(pDataOut) '
    WREG = pDataOut '
    _WriteSPI2

_WriteSPI2 Macro-
    GoSub __WriteSPI2
Endm

#ifMacro- _WriteSPI2
__WriteSPI2:
    SSP2STATbits_BF = 0             ' Empty the SSP2BUF buffer flags
    PIR3bits_SSP2IF = 0             ' Clear the interrupt flag
    SSP2BUF = WREG                  ' Place the data to send into SSP2BUF
    Btfss PIR3bits_SSP2IF           ' \
    Bra $ - 2                       ' / Wait until cycle complete
    WREG = SSP2BUF                  ' Read SSP2BUF into WREG
    Return
#endIfMacro-

'-----------------------------------------------------------------------------------
' Macro Name     : WriteReadSPI2
' Return Value   : Input data can be picked up by reading SSP2BUF straight after the WriteSPI2 macro
' Parameter      : Single 8-bit variable for SPI2 bus.
' Description    : Write and read a single byte (8-bits) to the SPI2 bus.
'
$define WriteReadSPI2(pDataOut) _WriteReadSPI2 pDataOut

_WriteReadSPI2 Macro- pDataOut
    #if (Prm_1 != Num8) && (Prm_1 != Byte)
        #error "_WriteReadSPI2 Parameter 1 must be constant value or a Byte"
        Exitm
    #else
        #if (Prm_1 == Byte)
        Byte_Wreg pDataOut
        #endif
        #if (Prm_1 == Num8)
        Num_Wreg pDataOut
        #endif        
    #endif
    GoSub __WriteReadSPI2_Sub
    return_Wreg       
Endm

#ifMacro- _WriteReadSPI2
__WriteReadSPI2_Sub:
    SSP2STATbits_BF = 0             ' Empty the SSP2BUF buffer flags
    PIR3bits_SSP2IF = 0             ' Clear the interrupt flag
    SSP2BUF = WREG                  ' Place the data to send into SSP2BUF
    Btfss PIR3bits_SSP2IF           ' \
    Bra $ - 2                       ' / Wait until cycle complete
    WREG = SSP2BUF                  ' Read SSP2BUF into WREG
    Return
#endIfMacro-

'-----------------------------------------------------------------------------------
' Macro Name    : ReadSPI2
' Return Value  : Contents of SSP2BUF register
' Parameters    : None
' Description   : Read byte\s from the SPI2 bus.
'
$define ReadSPI2() _ReadSPI2

_ReadSPI2 Macro- \Byte
    GoSub __ReadSPI2
    Return_Wreg
Endm


#ifMacro- _ReadSPI2
__ReadSPI2:
    WREG = SSP2BUF                  ' Empty the SSP2BUF buffer and clear flags
    PIR3bits_SSP2IF = 0             ' Clear interrupt flag
    SSP2BUF = $FF                   ' Initiate bus cycle
    Btfss PIR3bits_SSP2IF           ' \ 
    Bra $ - 2                       ' / Wait until cycle complete
    WREG = SSP2BUF                   ' Read SSP2BUF into WREG
    Return
#endIfMacro-

'-------------------------------------------------------------------------------
$define SPI2_Write(pDataOut) WriteSPI2(pDataOut)
$define SPI2_WriteRead(pDataOut) WriteReadSPI2(pDataOut)
$define SPI2_Read() ReadSPI2()
$define SPI2_Open(pSyncMode,pBusMode,pSmpPhase) OpenSPI2(pSyncMode,pBusMode,pSmpPhase)
$define SPI2_DataReady() SSP2STATbits_BF
$define SPI2_Close() SSP2CON1 = SSP2CON1 & $DF

$define DataReadySPI2() SSP2STATbits_BF
$define CloseSPI2() SSP2CON1 = SSP2CON1 & $DF

'-------------------------------------------------------------------------------
$endif
'-------------------------------------------------------------------------------
_SPI_MAIN_:
'-------------------------------------------------------------------------------
' Declare the Ansel pins to digital.
$ifdef CS0_Ansel_Pin    
    CS0_Ansel_Pin = 0
$endif        

$ifdef CS1_Ansel_Pin    
    CS1_Ansel_Pin = 0
$endif   

$ifdef CS2_Ansel_Pin    
    CS2_Ansel_Pin = 0
$endif   

$ifdef CS3_Ansel_Pin    
    CS3_Ansel_Pin = 0
$endif   

$ifdef CS4_Ansel_Pin    
    CS4_Ansel_Pin = 0
$endif       

' Set the Iddle State of the Chip Select pins
$ifdef SPI_CS0_Pin
    High SPI_CS0_Pin
$endif

$ifdef SPI_CS1_Pin
    High SPI_CS1_Pin
$endif

$ifdef SPI_CS2_Pin
    High SPI_CS2_Pin
$endif

$ifdef SPI_CS3_Pin
    High SPI_CS3_Pin
$endif

$ifdef SPI_CS4_Pin
    High SPI_CS4_Pin
$endif
'-------------------------------------------------------------------------------
$endif  ' __SPI_INC_
'===============================================================================
